import os
from pathlib import Path

import click

from djb.cli.click_util import project_dir_option
from djb.cli.constants import ERR, SUC


@click.command()
@project_dir_option()
def djbrc(project):
    """
    Update `.djbrc` script based on the project's current state.

    Tip: It's best to run `djb djbrc` as `djb djbrc && source .djbrc` to ensure
    that your shell environment reflects any changes.
    """
    click.echo("→ Updating .djbrc...")
    project_path = Path(project).resolve()
    djbrc_file_path = project_path / ".djbrc"

    # Update `.djbrc`
    djbrc_content = f"""\
# .djbrc

# Ensure .djbrc can only be sourced.
# This works because return only works inside a function or a sourced script.
( return 0 2>/dev/null ) || {{
    echo "$(basename "$0") must be sourced. Use: \\`. "$0"\\`" >&2
    exit 1
}}

# Exports.
export DJB_PROJECT_DIR={project_path}
"""
    try:
        with djbrc_file_path.open("w") as file:
            file.write(djbrc_content)
        click.echo(f"{SUC} Updated '{djbrc_file_path}'.")
    except Exception as e:
        click.echo(f"{ERR} Failed to update '{djbrc_file_path}': {e}", err=True)



import os

from typing import Any, Optional, Type

import click


class Off:
    def __init__(self, name="Off"):
        self.name = name

    def __bool__(self):
        return False
    
    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name


def project_dir_option():
    """Reusable option for specifying the djb project directory."""
    return click.option(
        "--project",
        type=click.Path(exists=True, file_okay=False, dir_okay=True, writable=True),
        default=os.getenv("DJB_PROJECT_DIR", "."),
        envvar="DJB_PROJECT_DIR",
        show_default=True,
        show_envvar=True,
        help="Path to djb project directory.",
        metavar="DIR",
    )



import os
import click

from djb.cli.click_util import project_dir_option
from djb.cli.djbrc import djbrc
from djb.cli.install import install


@click.command()
@project_dir_option()
@click.option(
    "--update",
    is_flag=True,
    help="""
    Update installed project dependencies and configure project environment.
    """,
)
@click.pass_context
def up(ctx, project, update):
    """
    Install development dependencies and configure project env.

    Shorthand for `djb install && djb djbrc`. Additionally, `djb up --update`
    is shorthand for `djb update && djb djbrc`.

    This command verifies and installs project dependencies, then updates
    the project's `.djbrc` script based on the current project state.

    Tip: It's best to run `djb up` as `djb up && source .djbrc` to ensure that
    your shell environment reflects any changes.
    """
    ctx.invoke(install, update=update)
    ctx.invoke(djbrc, project=project)



import click

from djb.cli.djbrc import djbrc
from djb.cli.install import install
from djb.cli.create import create
from djb.cli.up import up


class OrderedGroup(click.Group):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.command_order = []

    def add_command(self, cmd, name=None):
        super().add_command(cmd, name)
        self.command_order.append(name or cmd.name)

    def list_commands(self, ctx):
        return self.command_order


class AppContext:
    def __init__(self):
        # Global options.
        self.debug = False
        self.verbose = False

        # djb update options.
        self.update = False


@click.group(cls=OrderedGroup)
@click.option("--debug", is_flag=True, help="Enable debug output.")
@click.option("--verbose", is_flag=True, help="Enable verbose output.")
@click.pass_context
def cli(
    ctx,
    debug,
    verbose,
):
    """
    djb (dj_bun): playin' dev and deploy since 1984 🎶
    """
    ctx.ensure_object(AppContext)
    ctx.obj.debug = debug
    ctx.obj.verbose = verbose


cli.add_command(create)
cli.add_command(up)
cli.add_command(djbrc)
cli.add_command(install)


def main():
    cli(max_content_width=100)


if __name__ == "__main__":
    main()
